!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! TbTest.f90 - Main application module for TbTest sample. 
! 
! TbTest sample demonstrates usage of toolbars
!
! 2005. Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
MODULE TbTest

USE XFT
USE XFTToolbar
USE DFWIN, Unused => NULL
USE COMCTL32, Unused2 => NULL

IMPLICIT NONE

TYPE(X_MENU)::                xMenu                    !Main menu
TYPE(X_TOOLBAR)::             xTb
INTEGER::                     m_iColor = XCOLOR_YELLOW

!========================================================================
CONTAINS
!========================================================================
!PURPOSE: Initialization function called by XFT library on app start.
LOGICAL FUNCTION XInit(szCmdLine,nCmdShow)
!DEC$ATTRIBUTES DEFAULT, DECORATE, ALIAS: 'XINIT':: XInit

IMPLICIT NONE

INCLUDE 'Resource.fd'

CHARACTER(*), INTENT(IN)::    szCmdLine
INTEGER, INTENT(IN)::         nCmdShow
TYPE(X_TBBUTTON)::            xButton(11)
TYPE(X_BITMAP)::              xBmp
TYPE(T_IMAGEINFO)::           IIF

LOGICAL::            bSt
INTEGER::            i, hIl, hIcon, n
INTEGER, PARAMETER:: idIcon(3) = (/IDI_BLUE, IDI_YELLOW, IDI_RED/)


!Loading menu resource
xMenu = XLoadMenu(IDR_MENU_MAIN)

!Creation of App & frame window
bSt = XCreateSDIApp(xMenu, "TbTest", IDI_ICON_MAIN)

bSt = XSetHandler(XW_FRAME, WM_PAINT, XFrame_OnPaint)
bSt = XSetHandler(XW_FRAME, WM_SIZE, XW_FRAME_OnSize)

bSt = XSetCommand(XW_FRAME, IDM_HELP_ABOUT, XFrame_OnAbout)
bSt = XSetCommand(XW_FRAME, ID_BUTTON_CUT, XW_FRAME_OnButtonCut)

bSt = XSetCommand(XW_FRAME, ID_BUTTON_RED, XW_FRAME_OnButtonRBY)
bSt = XSetCommand(XW_FRAME, ID_BUTTON_BLUE, XW_FRAME_OnButtonRBY)
bSt = XSetCommand(XW_FRAME, ID_BUTTON_YELLOW, XW_FRAME_OnButtonRBY)

bSt = XSetCommand(XW_FRAME, ID_BUTTON_A, XW_FRAME_OnButtonABC)
bSt = XSetCommand(XW_FRAME, ID_BUTTON_B, XW_FRAME_OnButtonABC)
bSt = XSetCommand(XW_FRAME, ID_BUTTON_C, XW_FRAME_OnButtonABC)

bSt = XSetCommand(XW_FRAME, IDM_TOOLBAR_HORIZONTAL, XW_FRAME_OnToolbarChange)
bSt = XSetCommand(XW_FRAME, IDM_TOOLBAR_VERTICAL, XW_FRAME_OnToolbarChange)
bSt = XSetCommand(XW_FRAME, IDM_TOOLBAR_TOPLEFT, XW_FRAME_OnToolbarChange)
bSt = XSetCommand(XW_FRAME, IDM_TOOLBAR_BOTTOMRIGHT, XW_FRAME_OnToolbarChange)
bSt = XSetCommand(XW_FRAME, IDM_TOOLBAR_FLOATING, XW_FRAME_OnToolbarFloating)

bSt = XCreateToolbar(XW_FRAME, xTb, IDR_TOOLBAR1, TBSTYLE_TOOLTIPS.OR.TBSTYLE_FLAT.OR.CCS_DOCKABLE.OR.WS_BORDER)


!Create the image list for the items
hIL = ImageList_Create(16, 16, ILC_MASK, 3, 0)
DO i=1,3
      hIcon = LoadImage(XGetInstance(), idIcon(i), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR)
      bSt = ImageList_ReplaceIcon(hIL, -1, hIcon)
END DO
bSt = XToolbarSet(xTb, IDR_TOOLBAR1, hIL, CTL_BITMAP)

bSt = XCreateBitmap(xBmp, IDR_TOOLBAR1, LR_LOADMAP3DCOLORS)
bSt = XToolbarSet(xTb, IDR_TOOLBAR1, xBmp%hBmp, CTL_BITMAP)

bSt = XToolbarSet(xTb, IDR_TOOLBAR1, IDB_STD_SMALL_COLOR, CTL_BITMAP)

xButton(1) = X_TBBUTTON(ID_BUTTON_BLUE, TBSTATE_ENABLED, TBSTYLE_CHECKGROUP, 1, 0)
xButton(2) = X_TBBUTTON(ID_BUTTON_YELLOW, TBSTATE_ENABLED.OR.TBSTATE_CHECKED, TBSTYLE_CHECKGROUP, 2, 0)
xButton(3) = X_TBBUTTON(ID_BUTTON_RED, TBSTATE_ENABLED, TBSTYLE_CHECKGROUP, 3, 0)
xButton(4) = X_TBBUTTON(0, 0, TBSTYLE_SEP, 0, 0)
xButton(5) = X_TBBUTTON(ID_BUTTON_A, TBSTATE_ENABLED, TBSTYLE_BUTTON, 4, 0)
xButton(6) = X_TBBUTTON(ID_BUTTON_B, TBSTATE_ENABLED, TBSTYLE_BUTTON, 5, 0)
xButton(7) = X_TBBUTTON(ID_BUTTON_C, TBSTATE_ENABLED, TBSTYLE_BUTTON, 6, 0)
xButton(8) = X_TBBUTTON(0, 0, TBSTYLE_SEP, 0, 0)
xButton(9) = X_TBBUTTON(ID_BUTTON_CUT, TBSTATE_ENABLED, TBSTYLE_BUTTON,  7, 0)
xButton(10) = X_TBBUTTON(ID_BUTTON_COPY, TBSTATE_ENABLED, TBSTYLE_BUTTON,  8, 0)
xButton(11) = X_TBBUTTON(ID_BUTTON_PASTE, TBSTATE_ENABLED, TBSTYLE_BUTTON,  9, 0)

bSt = XToolbarAddButtons(xTb, xButton, 0)

bSt = XToolbarGet(xTb, IDR_TOOLBAR1, hIl, CTL_BITMAP)
n = ImageList_GetImageCount(hIl)
n = ImageList_GetImageInfo(hIl, 4, IIF)


XInit = .TRUE.

END FUNCTION XInit
!========================================================================
!Handler for painting in the main window. Does not use double-buffering by default.
LOGICAL FUNCTION XFrame_OnPaint(xWnd,xDC,iX1,iY1,iX2,iY2)

TYPE(X_WINDOW)::     xWnd              !Window
TYPE(X_DC)::         xDC               !Window's DC
INTEGER,INTENT(IN):: iX1,iY1,iX2,iY2   !Dimensions of update rectangle

LOGICAL::            bSt

bSt = XFillViewport(xDC, X_BRUSH(BS_SOLID, m_iColor, 0))
!TODO Add your custom drawing here
XFrame_OnPaint = .TRUE. 

END FUNCTION XFrame_OnPaint
!========================================================================
!Handler for File/Exit menu
SUBROUTINE XFrame_OnExit(xWnd,ID,nCode)

TYPE(X_WINDOW)::  xWnd
INTEGER::   ID, nCode

INTEGER::   iSt

iSt = XSendMessage(XW_FRAME, WM_CLOSE, 0, 0)

END SUBROUTINE XFrame_OnExit
!========================================================================
!Help/About handler
SUBROUTINE XFrame_OnAbout(xWnd,ID,nCode)

TYPE(X_WINDOW)::  xWnd
INTEGER::   ID, nCode

TYPE(X_WINDOW)::  xDlg
LOGICAL::         bSt
INTEGER::         iSt

INCLUDE "Resource.fd"

bSt = XLoadDialog(IDD_DIALOG_ABOUT, xDlg, xWnd)
iSt = XModalDialog(xDlg)
CALL XDestroyDialog(xDlg)

END SUBROUTINE XFrame_OnAbout
!=======================================================================
LOGICAL FUNCTION XW_FRAME_OnSize(xWnd, iWidth, iHeight, nFlag)
 
TYPE(X_WINDOW)::              xWnd
INTEGER, INTENT(IN)::         iWidth, iHeight
INTEGER, INTENT(IN)::         nFlag

INTEGER::                     iSt

iSt = XToolbarSetPos(xTb)
CALL XW_Frame_AdjustMenu()
XW_FRAME_OnSize=.FALSE.

END FUNCTION XW_FRAME_OnSize
!=======================================================================
SUBROUTINE XW_FRAME_AdjustMenu()

INTEGER::                     iStyle
LOGICAL::                     bSt

INCLUDE "Resource.fd"

bSt = XToolbarGet(xTb, IDR_TOOLBAR1, iStyle, CTL_STYLE)

bSt = XMenuSet(xMenu, IDM_TOOLBAR_HORIZONTAL, IAND(iStyle,CCS_VERT).EQ.0, XC_STATE)
bSt = XMenuSet(xMenu, IDM_TOOLBAR_VERTICAL, IAND(iStyle,CCS_VERT).NE.0, XC_STATE)

bSt = XMenuSet(xMenu, IDM_TOOLBAR_TOPLEFT, IAND(iStyle,CCS_BOTTOMRIGHT).EQ.0, XC_STATE)
bSt = XMenuSet(xMenu, IDM_TOOLBAR_BOTTOMRIGHT, IAND(iStyle,CCS_BOTTOMRIGHT).NE.0, XC_STATE)

bSt = XMenuSet(xMenu, IDM_TOOLBAR_FLOATING, IAND(iStyle,CCS_FLOATING).NE.0, XC_STATE)

END SUBROUTINE XW_FRAME_AdjustMenu
!=======================================================================
SUBROUTINE XW_FRAME_OnToolbarChange(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

LOGICAL::         bSt
INTEGER::         iStyle, idPair

INCLUDE "Resource.fd"

bSt = XToolbarGet(xTb, IDR_TOOLBAR1, iStyle, CTL_STYLE)
iStyle = iStyle .AND. NOT(CCS_FLOATING)   ! .AND. NOT(CCS_RIGHT)
SELECT CASE(idCmd)
CASE(IDM_TOOLBAR_HORIZONTAL)
      iStyle = iStyle .AND. NOT(CCS_VERT)
      idPair = IDM_TOOLBAR_VERTICAL
CASE(IDM_TOOLBAR_VERTICAL)
      iStyle = iStyle .OR. CCS_VERT
      idPair = IDM_TOOLBAR_HORIZONTAL
CASE(IDM_TOOLBAR_TOPLEFT)
      iStyle = iStyle .AND.NOT(CCS_BOTTOMRIGHT)
      idPair = IDM_TOOLBAR_BOTTOMRIGHT
CASE(IDM_TOOLBAR_BOTTOMRIGHT)
      iStyle = iStyle .OR. CCS_BOTTOMRIGHT
      idPair = IDM_TOOLBAR_TOPLEFT
END SELECT

bSt = XToolbarSet(xTb, IDR_TOOLBAR1, iStyle, CTL_STYLE)
CALL XW_FRAME_AdjustMenu()

END SUBROUTINE XW_FRAME_OnToolbarChange
!=======================================================================
SUBROUTINE XW_FRAME_OnToolbarFloating(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

LOGICAL::         bSt, bChecked
INTEGER::         iStyle, idPair

INCLUDE "Resource.fd"

bSt = XToolbarGet(xTb, IDR_TOOLBAR1, iStyle, CTL_STYLE)
IF (IAND(iStyle,CCS_FLOATING).EQ.0) THEN
      bSt = XToolbarSetPos(xTb, 20, 20)
ELSE
      iStyle = iStyle .AND. NOT(CCS_FLOATING)
      bSt = XToolbarSet(xTb, IDR_TOOLBAR1, iStyle, CTL_STYLE)
      bSt = XToolbarSetPos(xTb)
END IF
CALL XW_FRAME_AdjustMenu()

END SUBROUTINE XW_FRAME_OnToolbarFloating
!=======================================================================
SUBROUTINE XW_FRAME_OnButtonRBY(xWnd, idCmd, nCode)

TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

INCLUDE "Resource.fd"

IF (nCode.EQ.0) THEN
      SELECT CASE(idCmd)
      CASE(ID_BUTTON_BLUE)
            m_iColor = XCOLOR_BLUE
      CASE(ID_BUTTON_RED)
            m_iColor = XCOLOR_RED
      CASE(ID_BUTTON_YELLOW)
            m_iColor = XCOLOR_YELLOW
      END SELECT
END IF
CALL XUpdateWindow(XW_FRAME)

END SUBROUTINE XW_FRAME_OnButtonRBY
!=======================================================================
SUBROUTINE XW_FRAME_OnButtonABC(xWnd, idCmd, nCode)

TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

INCLUDE "Resource.fd"

INTEGER::      iBitmap
LOGICAL::      bSt

IF (nCode.EQ.0) THEN
      bSt=XToolbarGet(xTb, idCmd, iBitmap, CTL_BITMAP)
      iBitmap = iBitmap + 1
      IF (iBitmap.GT.6) iBitmap = 4
      bSt=XToolbarSet(xTb, idCmd, iBitmap, CTL_BITMAP)
END IF

END SUBROUTINE XW_FRAME_OnButtonABC
!=======================================================================
SUBROUTINE XW_FRAME_OnButtonCut(xWnd, idCmd, nCode)

USE XFTNotify
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

TYPE(T_NMTTDISPINFO), POINTER::     NMTD

INCLUDE "Resource.fd"

IF (nCode.EQ.TTN_GETDISPINFO) THEN
      NMTD => X_NMTTDISPINFO(nCode)
      NMTD%hinst = 0
      NMTD%lpszText = LOC("Button tooltip can be overriden by handling TTN_GETDISPINFO notification message"C)
END IF

END SUBROUTINE XW_FRAME_OnButtonCut
!========================================================================
END MODULE TbTest

